/* This file is part of GNU Dico.
   Copyright (C) 1998-2021 Sergey Poznyakoff

   GNU Dico is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   GNU Dico is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef __dico_h
#define __dico_h

#include <stdlib.h>
#include <stdarg.h>

#include <dico/types.h>
#include <dico/argcv.h>
#include <dico/list.h>
#include <dico/assoc.h>
#include <dico/stream.h>
#include <dico/url.h>
#include <dico/xlat.h>
#include <dico/strat.h>
#include <dico/utf8.h>
#include <dico/filter.h>
#include <dico/diag.h>
#include <dico/util.h>
#include <dico/parseopt.h>
#include <dico/markup.h>
#include <dico/udb.h>
#include <dico/tokenize.h>

#endif
    
